/* timeconversions.c routines to convert to and from time strings and
    seconds.
*/

/* to_timestring : converts sec.s to hh:mm:sec.s 
*/ 
#include "timeconversions.h"
void to_timestring(float secs, char *buf)
{
	int hh, mm;
        float ss, temp;

        hh = secs / 3600;
        temp = secs - (float) hh*3600;
        mm = temp / 60;
        ss = secs - (float) hh*3600 - (float) mm*60;
        if (hh == 0) {
          if( mm == 0){
			  sprintf(buf,"%.1f",ss);
		       }
          else  
	  if ( ss < 10.0 )
		sprintf(buf,"%d:0%.1f",mm,ss);
	  else sprintf(buf,"%d:%.1f",mm,ss);
          }
	else {
          if( mm == 0){
			if (ss < 10.0) 
				sprintf(buf,"%d:00%d:0%.1f",hh,ss); 
			 else sprintf(buf,"%d:00:%.1f",hh,ss);
		       }
          else  
	  if ( mm < 10 )
		if ( ss < 10.0 )
			sprintf(buf,"%d:0%d:0%.1f",hh,mm,ss);
		else    sprintf(buf,"%d:0%d:%.1f",hh,mm,ss);
	else 
		if ( ss < 10.0) 
			sprintf(buf,"%d:%d:0%.1f",hh,mm,ss);
		else
			sprintf(buf,"%d:%d:%.1f",hh,mm,ss);
	 }
	 
}

/* int to_seconds(char *timestring, char *secstring)

	converts timestring of format hrs:mins:sec.s into sec.s and
	stores resulting string in secstring. If timestring format
	is invalid, copies timestring up to first trailing whitespace
	to secstring. Returns 1 if valid timestring, 0 if invalid. Initial
	whitespace is copied. 
*/


int
to_seconds(char *timestring,char *secstring) 
{
	char buffer[BUFSIZE];
	float times[MAXFIELDS]; /* store (hh,mm,sec.s) */
	static float zeros[MAXFIELDS];
	char temp[BUFSIZE];
	int timeok = TRUE;        /* flag for invalid time string */
	int flds  = 0;
	int nxtchr;
	int i;
        int ctr = 0;
	int count = 0;
	int decptflg = FALSE; /* flag multiple decimal pts. */
	char  c;
	char *fmtstr=FMT;

	while(isspace(nxtchr=*timestring++))
	*secstring++ = nxtchr; /* copy initial whitespace */

	/* initialize hr,min,secs to zero */
	for(i=0;i<MAXFIELDS;i++)times[i]=zeros[i];

/*   parsing loop: acquire field and process it. */

	while (!((nxtchr == ' ')||(nxtchr =='\t')||(nxtchr == '\0'))) {
		buffer[ctr] = nxtchr; /* store all input */
		if (nxtchr == FLDSEP) {  /* dump next field from temp */
			if (timeok){
				temp[count] = '\0';
				times[flds] = atof(temp);
				++flds;
				count = 0;
		  	           }
		   }
		else {      /* gets done for numerical input */
			if (!((nxtchr >='0'&&nxtchr <='9')||
				(nxtchr == '.' && decptflg == FALSE)))
				timeok = FALSE;
			temp[count] = nxtchr;
			++count;
		     }	
		if (nxtchr == '.') decptflg = TRUE;
		nxtchr = *timestring++; /* continue to acquire field */
		++ctr;
	   } /* end of parsing while, field now complete */

	buffer[ctr] = '\0'; /* properly end strings */
	temp[count] = '\0';
	if (timeok) {
		times[flds] = atof(temp);           /* last one */
		sprintf(secstring,fmtstr,secs(times,flds+1));
	        } 
	else sprintf(secstring,"%s",buffer);

	return timeok;
			
}
				


/* secs:  convert (hrs,mins,sec.s) to sec.s
    secs(float time[], int nfields) */

float secs(float times[],int nfields)
{
	float s = 0.0;
	int count = 0;

	if (nfields > MAXFIELDS || nfields <0)
        return 0.0;	
        else {
		while (count + 1 <= nfields) {
			s = 60.0*s + times[count];
			++count;
 		      }
	}
	return s;
}
